
#include <stdlib.h>
#include "aspell.hh"
#include "language.hh"

void aspell::destroy() {
  WordLists::iterator i   = word_lists_.begin();
  WordLists::iterator end = word_lists_.end();
  for (; i != end; ++i) {
    if (i->own() && i->wl())
      delete i->wl();
  }
}

bool aspell::check_raw (const char *w) const {
  WordLists::const_iterator i   = word_lists_.begin();
  WordLists::const_iterator end = word_lists_.end();
  const StandardWL_Base * wl;
  for (; i != end; ++i) {
    if  (i->use_to_check() && 
	 (wl = dynamic_cast<const StandardWL_Base *>(i->wl())) &&
	 wl->operator[](w))
      return true;
  }
  return false;
};

bool aspell::check_raw (const string &w) const {
  WordLists::const_iterator i   = word_lists_.begin();
  WordLists::const_iterator end = word_lists_.end();
  const StandardWL_Base * wl;
  for (; i != end; ++i) {
    if  (i->use_to_check() && 
	 (wl = dynamic_cast<const StandardWL_Base *>(i->wl())) &&
	 wl->operator[](w))
      return true;
  }
  return false;
};

const aspell::SWordList 
  aspell::words_w_soundslike (const char *m) const 
{
  SWordList::outer_container con;
  SWordList::outer_container::value_type temp;
  WordLists::const_iterator i   = word_lists_.begin();
  WordLists::const_iterator end = word_lists_.end();
  const StandardWL_Base * wl;
  for (; i != end; ++i) {
    if  (i->use_to_suggest() && 
	 (wl = dynamic_cast<const StandardWL_Base *>(i->wl()))) {
      temp = wl->words_w_soundslike(m);
      if (temp.size()) con.push_back(temp);
    }
  }
  return SWordList().swap_in(con);
}

const aspell::SReplList 
  aspell::repls_w_soundslike (const char *m) const 
{
  SReplList::outer_container con;
  SReplList::outer_container::value_type temp;
  WordLists::const_iterator i   = word_lists_.begin();
  WordLists::const_iterator end = word_lists_.end();
  const ReplacementsWL_Base * wl;
  for (; i != end; ++i) {
    if  (i->use_to_suggest() && 
	 (wl = dynamic_cast<const ReplacementsWL_Base *>(i->wl()))) {
      temp = wl->words_w_soundslike(m);
      if (temp.size()) con.push_back(temp);
    }
  }
  return SReplList().swap_in(con);
}

void aspell::save_all_wls() {
  WordLists::iterator i   = word_lists_.begin();
  WordLists::iterator end = word_lists_.end();
  SavableWL_Base * wl;
  for (; i != end; ++i) {
    if  (i->save_on_saveall() && 
	 (wl = dynamic_cast<SavableWL_Base *>(i->wl())))
      wl->save();
  }
}

aspell::WordLists::size_type aspell::wl_size() const {
  return word_lists_.size();
}

bool aspell::wl_empty() const {
  return word_lists_.empty();
}

aspell::WordLists::iterator aspell::wl_begin() {
  return word_lists_.begin();
}

aspell::WordLists::const_iterator aspell::wl_begin() const {
  return word_lists_.begin();
}

aspell::WordLists::iterator aspell::wl_end() {
  return word_lists_.end();
}

aspell::WordLists::const_iterator aspell::wl_end() const {
  return word_lists_.end();
}

aspell::WordLists::iterator 
aspell::locate_wl(WordList_Base *to_find)
{
  WordLists::iterator   i = word_lists_.begin();
  WordLists::iterator end = word_lists_.end();
  while (i != end && i->wl() != to_find) ++i;
  return i;
}

aspell::WordLists::const_iterator 
aspell::locate_wl(WordList_Base *to_find) const
{
  WordLists::const_iterator   i = word_lists_.begin();
  WordLists::const_iterator end = word_lists_.end();
  while (i != end && i->wl() != to_find) ++i;
  return i;
}

aspell::WordLists::iterator 
aspell::locate_wl(aspell::SpecialId to_find)
{
  WordLists::iterator   i = word_lists_.begin();
  WordLists::iterator end = word_lists_.end();
  while (i != end && i->special_id_ != to_find) ++i;
  return i;
}

aspell::WordLists::const_iterator 
aspell::locate_wl(aspell::SpecialId to_find) const
{
  WordLists::const_iterator   i = word_lists_.begin();
  WordLists::const_iterator end = word_lists_.end();
  while (i != end && i->special_id_ != to_find) ++i;
  return i;
}

bool aspell::have_wl(WordList_Base *to_find) const {
  return locate_wl(to_find) != word_lists_.end();
}

bool aspell::have_wl(aspell::SpecialId to_find) const {
  return locate_wl(to_find) != word_lists_.end();
}

StandardWL_Base & aspell::main_wl() {
  return *static_cast<StandardWL_Base *>(locate_wl(main_id)->wl());
}

const StandardWL_Base & aspell::main_wl() const {
  return *static_cast<const StandardWL_Base *>(locate_wl(main_id)->wl());
}

WritableWL_Base & aspell::personal_wl() {
  return *static_cast<WritableWL_Base *>(locate_wl(personal_id)->wl());
}

const WritableWL_Base & aspell::personal_wl() const {
  return *static_cast<const WritableWL_Base *>(locate_wl(personal_id)->wl());
}

WritableWL_Base & aspell::session_wl() {
  return *static_cast<WritableWL_Base *>(locate_wl(session_id)->wl());
}

const WritableWL_Base & aspell::session_wl() const {
  return *static_cast<const WritableWL_Base *>(locate_wl(session_id)->wl());
}

ReplacementsWL_Base & aspell::personal_repl() {
  return *static_cast<ReplacementsWL_Base *>(locate_wl(per_repl_id)->wl());
}

const ReplacementsWL_Base & aspell::personal_repl() const {
  return *static_cast<const ReplacementsWL_Base *>(locate_wl(per_repl_id)->wl());
}

aspell::WordLists::iterator aspell::attach(WordList_Base *w) {
  WordLists::iterator i = locate_wl(w);
  if (i != word_lists_.end()) {
    return i;
  } else {
    if (!lang_) lang_ = new SC_Language(*w->lang());
    w->attach(*lang_);
    word_lists_.push_back(w);
    return word_lists_.end() - 1;
  }
}

bool aspell::detach(WordList_Base *w) {
  return detach(locate_wl(w));
}

bool aspell::detach(aspell::WordLists::iterator to_del) {
  if (to_del == word_lists_.end()) return false;
  to_del->wl()->detach();
  word_lists_.erase(to_del);
  return true;
}  

bool aspell::destroy(WordList_Base *w) {
  return destroy(locate_wl(w));
}


bool aspell::destroy(aspell::WordLists::iterator to_del) {
  if (to_del == word_lists_.end()) return false;
  if (!to_del->own()) throw ae_dont_own_wordlist(to_del->wl(), this);
  delete to_del->wl();
  word_lists_.erase(to_del);
  return true;
}

aspell::WordLists::iterator aspell::set_id(WordList_Base * w, SpecialId id) {
  return set_id(locate_wl(w), id);
}

aspell::WordLists::iterator aspell::set_id(aspell::WordLists::iterator to_change,
					   SpecialId id) {
  switch(id) {
  case main_id:
    if (!dynamic_cast<StandardWL_Base *>(to_change->wl()))
      throw ae_word_list_not_compellable(id, to_change, this);
    break;
  case personal_id:
  case session_id:
    if (!dynamic_cast<WritableWL_Base *>(to_change->wl()))
      throw ae_word_list_not_compellable(id, to_change, this);
    break;
  case per_repl_id:
    if (!dynamic_cast<ReplacementsWL_Base *>(to_change->wl()))
      throw ae_word_list_not_compellable(id, to_change, this);
    break;
  case none_id:
    break;
  }
  if (id != none_id && have_wl(id))
    throw ae_already_have_type(id, to_change, this);
  to_change->special_id_ = id;
  return to_change;
}


aspell::WordLists::iterator aspell::change_id(WordList_Base *w , SpecialId id) {
  return set_id(locate_wl(w), id);
}

aspell::WordLists::iterator aspell::change_id(aspell::WordLists::iterator to_change,
				      SpecialId id) {
  set_id(to_change, id);
  switch (id) {
  case main_id:
    to_change->use_to_check(true).use_to_suggest(true).save_on_saveall(false);
    break;
  case personal_id:
    to_change->use_to_check(true).use_to_suggest(true).save_on_saveall(true);
    break;
  case session_id:
    to_change->use_to_check(true).use_to_suggest(true).save_on_saveall(false);
    break;
  case per_repl_id:
    to_change->use_to_check(false).use_to_suggest(true).save_on_saveall(true);
    break;
  case none_id:
    break;
  }
  return to_change;
}



